module("Matrix", package.seeall)

vector = require "./Hump/vector"



Matrix = {}

Matrix.prototype = {	_11 = 0, _12 = 0, _13 = 0,
						_21 = 0, _22 = 0, _23 = 0,
						_31 = 0, _32 = 0, _33 = 0
		}

Matrix.mt = {}

function Matrix:New(s)
	return setmetatable(s, Matrix.mt)
end

Matrix.mt.__index =  Matrix.prototype

--Local to world transformation
function Matrix.prototype:WorldTransform(BotVectorTable, position, forward, side)

	self:Rotate(forward, side)

	self:Translate(position.x, position.y)

	return self:TransformVector(BotVectorTable)
end

--Multiplies two martices
function Matrix.prototype:Mult(mat)
	--First row
	self._11 = (self._11 * mat._11) + (self._12 * mat._21) + (self._13 * mat._31)
	self._12 = (self._11 * mat._12) + (self._12 * mat._22) + (self._13 * mat._32)
	self._13 = (self._11 * mat._13) + (self._12 * mat._23) + (self._13 * mat._33)

	--Second Row
	self._21 = (self._21 * mat._11) + (self._22 * mat._21) + (self._23 * mat._31)
	self._22 = (self._21 * mat._12) + (self._22 * mat._22) + (self._23 * mat._32)
	self._23 = (self._21 * mat._13) + (self._22 * mat._23) + (self._23 * mat._33)

	--Third Row
	self._31 = (self._31 * mat._11) + (self._32 * mat._21) + (self._33 * mat._31)
	self._32 = (self._31 * mat._12) + (self._32 * mat._22) + (self._33 * mat._32)
	self._33 = (self._31 * mat._13) + (self._32 * mat._23) + (self._33 * mat._33)
end

--Takes a table of vectors that represent a bots vertices and Multiplies them
--By a matrix to transform the vectors, the matrix would be already setup by
--Setting it to an Identity matrix, set the scale, rotation or translate values
function Matrix.prototype:TransformVector(LocalVectorTable)
	tempVectorTable = {vector.new(), vector.new(), vector.new()}
	for i = 1, 3 do
		tempVectorTable[i].x = (self._11 * LocalVectorTable[i].x) + (self._21 * LocalVectorTable[i].y) + (self._31)
		tempVectorTable[i].y = (self._12 * LocalVectorTable[i].x) + (self._22 * LocalVectorTable[i].y) + (self._32)
	end

	return tempVectorTable
end

--Creates an Identity matrix
function Matrix.prototype:Identity()
	self._11 = 1
	self._12 = 0
	self._13 = 0
	self._21 = 0
	self._22 = 1
	self._23 = 0
	self._31 = 0
	self._32 = 0
	self._33 = 1
end

--Create a transformation matrix
function Matrix.prototype:Translate(x, y)
	mat = Matrix:New{
		_11 = 1, _12 = 0, _13 = 0,
		_21 = 0, _22 = 1, _23 = 0,
		_31 = x, _32 = y, _33 = 1
	}

	self:Mult(mat)
end

--Create a scale matrix
function Matrix.prototype:Scale(xScale, yScale)
	mat = Matrix:New{
		_11 = xScale, _12 = 0, _13 = 0,
		_21 = 0, _22 = yScale, _23 = 0,
		_31 = 0, _32 = 0, _33 = 1
	}

	self:Mult(mat)
end

--Create a rotation matrix from a 2D vector
function Matrix.prototype:Rotate(fwd, side)
		mat = Matrix:New{
		_11 = fwd.x, _12 = fwd.y, _13 = 0,
		_21 = side.x, _22 = side.y, _23 = 0,
		_31 = 0, _32 = 0, _33 = 1
	}

	self:Mult(mat)
end

--Print the matrix for debugging
function Matrix.prototype:Print()
	print(self._11, self._12, self._13)
	print(self._21, self._22, self._23)
	print(self._31, self._32, self._33)
end



