Vector = require "./Hump/vector"

--Module: SteeringBehaviour
module("SteeringBehaviour", package.seeall)

Steering = {}

--Member variables
Steering.prototype = {
	seekWeight = .9,
	arriveWeight = .9,
	fleeWeight = .7,
	pursuitWeight = .6,
	DecelerationType = {Slow = 3, Normal = 2, Fast = 1}
	}

Steering.mt = {}

function Steering:New(object)
	return setmetatable(object, Steering.mt)
end

Steering.mt.__index =  Steering.prototype

--Steering Functions
function Steering.prototype:Seek(bot, target)
	local Desiredvelocity = vector.new()

	Desiredvelocity = (target - bot.position):normalized() * bot.maxSpeed

	return (Desiredvelocity - bot.velocity)
end

function Steering.prototype:Flee(bot, target)
	local Desiredvelocity = vector.new(0,0)

	Desiredvelocity = (bot.position - target):normalized() * bot.maxSpeed

	return (Desiredvelocity - bot.velocity)
end

function Steering.prototype:Arrive(bot, targetPosition)
	ToTarget = vector.new()
	Desiredvelocity = vector.new()
	DecelerationType = self.DecelerationType.Slow

	ToTarget = targetPosition - bot.position

	dist = ToTarget:len()

	if dist > 0 then
		decelerationTweaker = 0.7

		speed = dist / (DecelerationType * decelerationTweaker)

		if speed > bot.maxSpeed then
			speed = bot.maxSpeed
		end

		Desiredvelocity = ToTarget* speed / dist

		return (Desiredvelocity - bot.velocity)
	end

	return vector.new(0, 0)
end

function Steering.prototype:Pursuit(Pursuer, Evader)
	ToEvader = vector.new()

	ToEvader = Evader.position - Pursuer.position

	RelativeHeading = Pursuer.heading * Evader.heading

	if (ToEvader * Pursuer.heading) > 0 and
		(RelativeHeading < -0.95) then
		return self:Seek(Pursuer, Evader.position)
	end

	LookAheadTime = ToEvader:len() / (Pursuer.maxSpeed / Evader.maxSpeed)

	return self:Seek(Pursuer, Evader.position + Evader.velocity * LookAheadTime)
end


--This calculates the steering force for the  bot
function Steering.prototype:CalculateSteeringForce(robot, evader, target)
	SteeringForce = vector.new(0,0)
	if robot.seekOn == true then
		SteeringForce = SteeringForce + self:Seek(robot, target) * self.seekWeight
	end

	if robot.fleeOn == true then
		SteeringForce = SteeringForce + self:Flee(robot, target) * self.fleeWeight
	end

	if robot.arriveOn == true then
		SteeringForce = SteeringForce + self:Arrive(robot, target) * self.arriveWeight
	end

	if robot.pursuitOn == true then
		SteeringForce = SteeringForce + self:Pursuit(robot, evader) * self.pursuitWeight
	end

	return SteeringForce
end

