vector = require "./Hump/vector"
require "Vehicle"
require "SteeringBehaviour"
require "Matrix"

--Create Objects
Bot1 = Vehicle.Bot:New{}
Bot2 = Vehicle.Bot:New{}
SB = SteeringBehaviour.Steering:New{}
B1 = Matrix.Matrix:New{}

--Table used to set the bot size in local space, this established a triangle
BotLocalCordSize = {vector.new(32,0), vector.new(0, 15), vector.new(0, -15)}

--Grab the mouse location
function updateMouseLoc()
	mousePosition = vector.new(0,0)
	mousePosition.x = love.mouse.getX()
	mousePosition.y = love.mouse.getY()
end

--Love2D callback functions
function love.load()
	--Initialize bot values
	Bot1:Init(vector.new(400,300))
	Bot1:SetArrive(true)
	Bot2:Init(vector.new(100,100))
	Bot2:SetPursuit(true)
end

function love.update(dt)
	updateMouseLoc()

	--Update the bots
	Bot1:Update(dt, SB, Bot2, mousePosition)
	Bot2:Update(dt, SB, Bot1, mousePosition)
end

function love.draw()
	--Drawing some ground
    love.graphics.setColor(207, 210, 200, 255)
	love.graphics.rectangle("fill", 0, 0, 800, 600)

	--Setting up the transform for Bot1 to draw a triangle
	B1:Identity()

	--Pass in vectors with the bot size in local space, it's position, heading (same as velocity), and side (perpendicular to heading)
	Bot1TriangleCoords = B1:WorldTransform(BotLocalCordSize, Bot1.position, Bot1.heading, Bot1.side)

	--Some print functions for debugging
	love.graphics.setColor(80, 100, 160, 255)
	love.graphics.print("Bot1 Vertices: " ..Bot1TriangleCoords[1].x..", "..Bot1TriangleCoords[1].y, 25, 25)
	love.graphics.print("Bot1 Vertices: " ..Bot1TriangleCoords[2].x..", "..Bot1TriangleCoords[2].y, 25, 45)
	love.graphics.print("Bot1 Vertices: " ..Bot1TriangleCoords[3].x..", "..Bot1TriangleCoords[3].y, 25, 65)
	--love.graphics.print("Bot1 Side: " ..Bot1.side.x..", "..Bot1.side.y, 25, 45)
	--love.graphics.print("Bot1 Heading: " ..Bot1.heading.x..", "..Bot1.heading.y, 25, 25)
	--love.graphics.print("Bot1 Side: " ..Bot1.side.x..", "..Bot1.side.y, 25, 45)

	--Draw the first bot's triangle with the points returned by WorldTransform
	love.graphics.setColor(80, 100, 160, 255)
	love.graphics.triangle("fill", Bot1TriangleCoords[1].x, Bot1TriangleCoords[1].y, Bot1TriangleCoords[2].x, Bot1TriangleCoords[2].y, Bot1TriangleCoords[3].x, Bot1TriangleCoords[3].y)
	--love.graphics.rectangle("fill", Bot1.position.x, Bot1.position.y, 32, 32)

	--Draw the other bot
	love.graphics.setColor(160, 100, 80, 255)
	love.graphics.rectangle("fill", Bot2.position.x, Bot2.position.y, 32, 32)
end

